#!/usr/bin/perl -w

use lib "/opt/andrea/bin/";

use strict;
use DBI;
use loadconfig;

readconfig;

my $DB_DSN      = "DBI:mysql:database=$loadconfig::conf{'DB_DATABASE'};host=$loadconfig::conf{'DB_HOST'}";
my $DB_USER     = $loadconfig::conf{'DB_USER'};
my $DB_PASSWD   = $loadconfig::conf{'DB_PASSWD'};
my $TABLE       = "andrea";

my $bn_subnet = $loadconfig::conf{'BN_SUBNET'};
my $bn_netmask = $loadconfig::conf{'BN_NETMASKL'};
my $bn_or = $loadconfig::conf{'BN_ROUTER'};
my $bn_odns = $loadconfig::conf{'BN_DNS'};
my $bn_odn = $loadconfig::conf{'BN_DN'};
my $bn_poolrange = $loadconfig::conf{'BN_POOLRANGE'};
my $bn_minlease = $loadconfig::conf{'BN_MINLEASE'};
my $bn_maxlease = $loadconfig::conf{'BN_MAXLEASE'};
my $rn_subnet = $loadconfig::conf{'RN_SUBNET'};
my $rn_netmask = $loadconfig::conf{'RN_NETMASKL'};
my $rn_or = $loadconfig::conf{'RN_ROUTER'};
my $rn_odns = $loadconfig::conf{'RN_DNS'};
my $rn_odn = $loadconfig::conf{'RN_DN'};
my $rn_poolrange = $loadconfig::conf{'RN_POOLRANGE'};
my $rn_minlease = $loadconfig::conf{'RN_MINLEASE'};
my $rn_maxlease = $loadconfig::conf{'RN_MAXLEASE'};
my $rn_wins = $loadconfig::conf{'RN_WINS'};
my $wins = "";

my $dhcp_outfile = "/etc/dhcp3/dhcpd.conf.new";
my $outtxt = "";
my $i;

my $dbh = DBI->connect($DB_DSN, $DB_USER,
   $DB_PASSWD) or die "Cannot connect to DB";


my $sql = qq[
             SELECT
             ipadr, macadr, nickname
             FROM $TABLE
             WHERE macadr!="" AND (dhcpoff!=1 OR dhcpoff IS NULL)
            ];
my $sth = $dbh->prepare($sql);
$sth->execute();
my $numRows = $sth->rows;

while (my @row = $sth->fetchrow_array ) {
 my ($ipadr, $macadr, $name);
 $ipadr  = $row[0];
 $macadr = $row[1];
 $name   = $row[2];
 if(length($macadr) == 17) {
  #push @outtxt, qq~    host $name {hardware ethernet $macadr; fixed-address $ipadr; }\n~;
  $outtxt .= qq~    host $name {hardware ethernet $macadr; fixed-address $ipadr; }\n~;
 $i++;
 }
}


if($rn_wins) {
 $wins = qq~
  option netbios-name-servers     $rn_wins;
  option netbios-node-type        8;
  option netbios-scope            "";
 ~;
}

my $dhcp_start = qq~
ddns-update-style none;
authoritative;
deny duplicates;

shared-network $rn_odn {

 subnet $rn_subnet netmask $rn_netmask {

  option routers                $rn_or;
  option domain-name-servers    $rn_odns;
  $wins
  option domain-name              "$rn_odn";

  pool {
   authoritative;
   deny unknown-clients;
   default-lease-time              $rn_minlease;  
   max-lease-time                  $rn_maxlease; 
   range $rn_poolrange;
~;

my $dhcp_end = qq~

  }

 }

 subnet $bn_subnet netmask $bn_netmask {

  option routers                $bn_or;
  option domain-name-servers    $bn_odns;
  option domain-name              "$bn_odn";

  pool {
    authoritative;
    deny dynamic bootp clients;
    default-lease-time             $bn_minlease;    # seconds
    max-lease-time                 $bn_maxlease;   # seconds
    range $bn_poolrange;
  }
 }

}
~;

my $ausgabe = $dhcp_start.$outtxt.$dhcp_end;
open (DATA, ">$dhcp_outfile");
print DATA $ausgabe;
close(DATA);

