#!/usr/bin/perl -w

use lib "/opt/andrea/bin/";

use strict;
use DBI;
use loadconfig;

readconfig;

my $DB_DSN      = "DBI:mysql:database=$loadconfig::conf{'DB_DATABASE'};host=$loadconfig::conf{'DB_HOST'}";
my $DB_USER     = $loadconfig::conf{'DB_USER'};
my $DB_PASSWD   = $loadconfig::conf{'DB_PASSWD'};
my $TABLE       = "andrea";

my $bn_subnet = $loadconfig::conf{'BN_SUBNET'};
my $bn_netmask = $loadconfig::conf{'BN_NETMASKL'};
my $bn_or = $loadconfig::conf{'BN_ROUTER'};
my $bn_odns = $loadconfig::conf{'BN_DNS'};
my $bn_odn = $loadconfig::conf{'BN_DN'};
my $bn_poolrange = $loadconfig::conf{'BN_POOLRANGE'};
my $bn_minlease = $loadconfig::conf{'BN_MINLEASE'};
my $bn_maxlease = $loadconfig::conf{'BN_MAXLEASE'};
my $rn_subnet = $loadconfig::conf{'RN_SUBNET'};
my $rn_netmask = $loadconfig::conf{'RN_NETMASKL'};
my $rn_or = $loadconfig::conf{'RN_ROUTER'};
my $rn_odns = $loadconfig::conf{'RN_DNS'};
my $rn_odn = $loadconfig::conf{'RN_DN'};
my $rn_poolrange = $loadconfig::conf{'RN_POOLRANGE'};
my $rn_minlease = $loadconfig::conf{'RN_MINLEASE'};
my $rn_maxlease = $loadconfig::conf{'RN_MAXLEASE'};
my $rn_dnsforward = $loadconfig::conf{'RN_DNSFORWARD'};

my $dbh = DBI->connect($DB_DSN, $DB_USER,
   $DB_PASSWD) or die "Cannot connect to DB";

my $dhcp_outfile = "/etc/bind/db.real.net.new";
my $dhcp_infile = "/etc/bind/db.real.net";
my $named_outfile = "/etc/bind/named.andrea.conf";
my $outtxt = "";
my $rvsouttxt = "";
my ($i, $serial, $now, $alias, @alias);

my ($sec, $min, $hour, $day, $month, $year, undef, undef, undef) = localtime(time());
$year += 1900;
$month += 1;
if (length($month)<=1) { $month = "0".$month; }
if (length($day)<=1) { $day= "0".$day; }
$now = 1;

if( -f $dhcp_infile) {
 open (DATA, "<$dhcp_infile");
 my @data = <DATA>;
 close(DATA);

 foreach my $temp (@data) {
  my $typ = "";
  my $ser = "";
  $temp =~ s/ //gim;
  ($ser, $typ) = split(/;/, $temp);
  if($typ and substr($typ,0,6) eq "serial") {
   $now = $ser;
   $now++;
  }
 }
} else {
 $now = 1;
}

if($rn_dnsforward) {
 $rn_dnsforward = qq~
	forwarders { $rn_dnsforward; };
	#forward first;
 ~;
}
my($rn1, $rn2, $rn3, undef) = split(/\./, $rn_subnet);

my $named_conf = qq~
zone "$rn_odn" in {
	type master;
	file "/etc/bind/db.real.net";
};
zone "$rn3.$rn2.$rn1.in-addr.arpa" in {
        type master;
        file "/etc/bind/db.$rn3.$rn2.$rn1.zone";
};
~;

my $rvsout_file = "/etc/bind/db.$rn3.$rn2.$rn1.zone";

my $dnsheader = qq~
; Zone file for $rn_odn
;
; The full zone file
;
\$TTL 3D
\@               IN SOA          ns1.$rn_odn. admin.$rn_odn. (
                                $now      ; serial
                                4H              ; refresh
                                1H              ; retry
                                2W              ; expiry
                                1D )            ; minimum
                TXT             "$rn_odn "
		NS		ns1.$rn_odn.

ns1		A		$rn_odns
~;

my $rvsheader = qq~
\$TTL 1D
\@               IN SOA          $rn_odn.   admin.$rn_odn. (
                                $now            ; serial (d. adams)
                                4H              ; refresh
                                1H              ; retry
                                2W              ; expiry
                                1D )            ; minimum
                IN NS           ns1.$rn_odn.
~;

my $sql = qq[
             SELECT
             ipadr, nickname, alias
             FROM $TABLE
            ];
my $sth = $dbh->prepare($sql);
$sth->execute();
my $numRows = $sth->rows;

while (my @row = $sth->fetchrow_array ) {
 my ($ipadr, $name);
 $ipadr    = $row[0];
 my($ip1, $ip2, $ip3, $ip4) = split(/\./, $ipadr);
 $name     = $row[1];
 $alias    = $row[2];
 $outtxt .= qq~$name\t\tA\t$ipadr\n~;
 $rvsouttxt .= qq~$ip4\t\tPTR\t$name\n~;
 if($alias){
  @alias = split(/,/, $alias);
  foreach my $temp (@alias) {
   $temp =~ tr/0-9a-zA-Z//cs;
   $outtxt .= qq~$temp\t\tA\t$ipadr\n~;
  }
 }
}

if( -f $dhcp_infile) {
 open (DATA, ">$dhcp_outfile");
 print DATA $dnsheader.$outtxt;
 close(DATA);
} else {
 open (DATA, ">$dhcp_infile");
 print DATA $dnsheader.$outtxt;
 close(DATA);
}

open (DATA, ">$rvsout_file");
print DATA $rvsheader.$rvsouttxt;
close(DATA);

open (DATA, ">$named_outfile");
print DATA $named_conf;
close(DATA);

